% Set coefficients & coefficient functions in SDE's
% based on TyrsFull time grid
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%% 1.Set Values on Time Grids %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%--- Volatilities ---%
xi = zeros(NtFull+1,1);   % KRW HW Vol:(t)
psi = zeros(NtFull+1,1);  % USD HW Vol:(t)
sig = zeros(NtFull+1,1);  % FX(KRW/USD):(t)
for k = 1:NtFull+1
    xi(k) = interpol_Const(TdysHW_KRW, HWVols_KRW, TdysFull(k));
    psi(k) = interpol_Const(TdysHW_USD, HWVols_USD, TdysFull(k));
    sig(k) = FXVol_KRWUSD;
end

%--- (t) ---%
% (1) Current Zero Curve
ZeroR_KRW = zeros(NtFull+1,1);
for k = 1:NtFull+1
    ZeroR_KRW(k) = interpol_Lin(TdysZC_KRW, RatesZC_KRW, TdysFull(k));
end

% (2) Current Instantaneous Forward Rate (Average Value)
f0_KRW = zeros(NtFull,1);
for k = 1:NtFull
    % Average in [t(k), t(k+1]
    f0_KRW(k) = (ZeroR_KRW(k+1)*TyrsFull(k+1) ...
              - ZeroR_KRW(k)*TyrsFull(k))/ht(k);
end

% (3) For Numerical Integration
B = zeros(NtFull-1,1);     % B(t,T)^2
SumB = zeros(NtFull,1);    % (t(n))  (t)^2*d[B(t,T)^2]
SumB(1) = 0;
SumB(2) = -xi(2)^2*getBT_HW1F(a, TyrsFull(1), TyrsFull(2))^2;
for n=3:NtFull
    for k=1:n-1
        B(k) = getBT_HW1F(a, TyrsFull(k), TyrsFull(n))^2;
    end

    for k=1:n-2
        SumB(n) = SumB(n) + xi(k+1)^2*(B(k+1) - B(k));
    end
        SumB(n) = SumB(n) - xi(n)^2*B(n-1);
end

% (4) (t)
alpha = zeros(NtFull,1);
alpha(1) = f0_KRW(1);
for n=2:NtFull
     alpha(n) = f0_KRW(n) - SumB(n)/2;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%% 2.Set Intermediate Values %%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--- Time ---%
Tdys_mid = zeros(NtFull+1,1);
for k = 1:NtFull
    Tdys_mid(k) = (TdysFull(k)+TdysFull(k+1))/2;
end
Tdys_mid(NtFull+1) = Tdys_mid(NtFull) + htDay;
Tyrs_mid = Tdys_mid/365;

ht_mid = zeros(Nt,1);
ht_mid = Tyrs_mid(2:NtFull+1) - Tyrs_mid(1:NtFull);

%--- Volatility ---%
xi_mid = zeros(Nt+1,1);     % KRW HW Vol:(t)
psi_mid = zeros(Nt+1,1);    % USD HW Vol:(t)
for k = 1:NtFull+1
    xi_mid(k) = interpol_Const(TdysHW_KRW, HWVols_KRW, Tdys_mid(k));
    psi_mid(k) = interpol_Const(TdysHW_USD, HWVols_USD, Tdys_mid(k));
end

%--- (t) ---%
% (1) Current Zero Curve
ZeroR_KRW_mid = zeros(NtFull+1,1);
for k = 1:Nt+1
    ZeroR_KRW_mid(k) = interpol_Lin(TdysZC_KRW, RatesZC_KRW, Tdys_mid(k));
end

% (2) Current Instantaneous Forward Rate (Average Value)
f0_KRW_mid = zeros(NtFull,1); % f0_KRW_mid(Nt+1) is not used.
for k = 1:NtFull
    f0_KRW_mid(k) = (ZeroR_KRW_mid(k+1)*Tyrs_mid(k+1) ...
                  - ZeroR_KRW_mid(k)*Tyrs_mid(k))/ht_mid(k);
end

% (3) For Numerical Integration
B_mid = zeros(NtFull-1,1);     % B(t,T)^2
SumB_mid = zeros(NtFull,1);    % (t(n))  (t)^2*d[B(t,T)^2]
SumB_mid(1) = 0;
SumB_mid(2) = -xi_mid(2)^2*getBT_HW1F(a, Tyrs_mid(1), Tyrs_mid(2))^2;
for n=3:NtFull
    for k=1:n-1
        B_mid(k) = getBT_HW1F(a, Tyrs_mid(k), Tyrs_mid(n))^2;
    end

    for k=1:n-2
        SumB_mid(n) = SumB_mid(n) ...
                     + xi_mid(k+1)^2*(B_mid(k+1) - B_mid(k));
    end
        SumB_mid(n) = SumB(n) - xi_mid(n)^2*B_mid(n-1);
end

% (4) (t)
alpha_mid = zeros(NtFull,1);
alpha_mid(1) = f0_KRW_mid(1);
for n=2:NtFull
     alpha_mid(n) = f0_KRW_mid(n) - SumB_mid(n)/2;
end
