% Solve FDM for Zero Bond Option (Implicit OSM. u_old => u => u_new)

% Terminal Condition
Uold = zeros(Nx+1, Ny+1);
Unew = zeros(Nx+1, Ny+1);
ZCB_KRW = zeros(Nx+1, Ny+1);
ZCB_USD = zeros(Nx+1, Ny+1);
for i=1:Nx+1
    for j=1:Ny+1
        ZCB_KRW(i,j) = getZCB_HW1F(TyrsZC_KRW, RatesZC_KRW, a, xi, x(i), ...
              TyrsFull, ht, Nt+1, S1);
        ZCB_USD(i,j) = getZCB_HW1F(TyrsZC_USD, RatesZC_USD, b, psi, y(j), ...
              TyrsFull, ht, Nt+1, S2);
        Unew(i,j) = (ZCB_KRW(i,j)>Strike_KRW)*(ZCB_USD(i,j)>Strike_USD);
    end
end

% Backwardation : U(k+1) => tmpU => U(k) => tmpU => U(k-1) => ...
A = zeros(Nx-1,Nx-1);   % Coefficient Matrix for A*Ux=gx
B = zeros(Ny-1,Ny-1);   % Coefficient Matrix for B*Uy=gy
gx = zeros(Nx+1,1);   % Source for A*Ux=gx. gx(1), gx(Nx+1) are not used.
gy = zeros(Ny+1,1);   % Source for B*Uy=gy. gy(1), gy(Ny+1) are not used.

for k = Nt:-1:1    % For each time grid (k=1 corresponds to valuation date.)   
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%% 1.A*Ux=gx : Uy => Ux (U(k+1) => tmpU) %%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % (1)Update Coefficient Matrix
    for i=2:Nx-2    % For each x(i)
        A(i,i-1) = dx(k,i+1);
        A(i,i) = ex(k);
        A(i,i+1) = fx(k,i+1);
    end 
        % Apply Linear B.C
        A(1,1) = ex(k) + 2*dx(k,2);
        A(1,2) = fx(k,2) - dx(k,2);
        A(Nx-1,Nx-2) = dx(k,Nx) - fx(k,Nx);
        A(Nx-1,Nx-1) = ex(k) + 2*fx(k,Nx);

    % (2)Solve A*Ux=gx (Implicit in x)
    Uold = Unew;    % Update Source
    for j=2:Ny % For each interior y(j)
        % Update (RHS)
        for i=2:Nx % % For each interior x(i)
            gx(i) = sx(k)*(Uold(i+1,j+1)-Uold(i+1,j-1)-Uold(i-1,j+1) ...
                  + Uold(i-1,j-1)) + (1-wx(k,i))*Uold(i,j);
        end

        % Solve Unew
        Unew(2:Nx,j) = A\gx(2:Nx);
        Unew(1,j) = 2*Unew(2,j) - Unew(3,j);   % at x=xmin
        Unew(Nx+1,j) = 2*Unew(Nx,j) - Unew(Nx-1,j);   % at x=xmax
    end

    % (3)Get Boundary Values (Linear B.C)   
    for i=1:Nx+1
       Unew(i,1) = 2*Unew(i,2) - Unew(i,3); % at y=ymin
       Unew(i,Ny+1) = 2*Unew(i,Ny) - Unew(i,Ny-1);  % at y=ymax
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%% 2.B*Uy=gy : Ux => Uy (tmpU => U(k)) %%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % (1)Update Coefficient Matrix
    for j=2:Ny-2 % For each interior y(j)
       B(j,j-1) = dy(k,j+1);
       B(j,j) = ey(k);
       B(j,j+1) = fy(k,j+1);
    end
        % Apply Linear B.C
        B(1,1) = ey(k) + 2*dy(k,2);
        B(1,2) = fy(k,2) - dy(k,2);
        B(Ny-1,Ny-2) = dy(k,Ny) - fy(k,Ny);
        B(Ny-1, Ny-1) = ey(k) + 2*fy(k,Ny);
    
    % (2)Solve A*Uy=gy (Implicit in y)
    Uold = Unew;    % Update Source
    for i=2:Nx % For each interior x(i)
        % Update (RHS)
        for j=2:Ny  % For each interior y(j)
            gy(j) = sy(k)*(Uold(i+1,j+1)-Uold(i+1,j-1)-Uold(i-1,j+1) ...
                  + Uold(i-1,j-1)) + (1-wy(k,i))*Uold(i,j);
        end

        % Solve Unew
        Unew(i,2:Ny) = (B\gy(2:Ny))';
        Unew(i,1) = 2*Unew(i,2) - Unew(i,3); % at y=ymin
        Unew(i,Ny+1) = 2*Unew(i,Ny) - Unew(i,Ny-1);  % at y=ymax
    end

    % (3)Get Boundary Value (Linear B.C)
    for j=1:Ny+1
        Unew(1,j) = 2*Unew(2,j) - Unew(3,j);   % at x=xmin
        Unew(Nx+1,j) = 2*Unew(Nx,j) - Unew(Nx-1,j);   % at x=xmax
    end
end

ZBDC2d_FDM(kkk) = Unew(Nx/2+1, Ny/2+1);

