function ZBC = getZBC_HW1F(TyrsZC, RatesZC, a, sig, x, TyrsFull, ht, ...
                  m, T, S, X)
% Calculate zero bond option price ZBC(t,T,S,X) under HW1F with a & (t)

% P(t,T) = exp{A(t,T) - x(t)*B(t,T)}
% TdysZC, RatesZC : Zero Curve
% a : Mean reversion
% sig : (t) vector
% x : x(i)
% TyrsFull : Time vector
% ht : dt vector
% m : Calculation time index. t=TyrsFull(m).
% T : Maturity of option
% S : Maturity of underlying zero bond (T<S)

% Numerical Integration
Integral = 0;
n = getIndex(TyrsFull, m, T);  % smallest index n s.t. TyrsFull(n)T.
for k=m:n-2
    Integral = Integral + sig(k+1)^2 ...
               *(exp(2*a*TyrsFull(k))+exp(2*a*TyrsFull(k+1)))/2;
end
Integral = Integral + sig(n)^2*(exp(2*a*TyrsFull(n-1))+exp(2*a*T))/2;

% Input Variables
sigT = sqrt((exp(-a*S)-exp(-a*T))^2*Integral);
PT = getZCB_HW1F(TyrsZC, RatesZC, a, sig, x, TyrsFull, ht, m, T);
PS = getZCB_HW1F(TyrsZC, RatesZC, a, sig, x, TyrsFull, ht, m, S);
d1 = (log(PS/PT)-log(X))/sigT + sigT/2;
d2 = d1 - sigT;

% Option Value
ZBC = PS*CND(d1) - X*PT*CND(d2);

function CND = CND(z)
% CDF for N(0,1)
CND = cdf('norm', z, 0, 1);
