function ZBDC2d = getZBDC2d_HW1F(TyrsZC1, RatesZC1, a1, sig1, x, ...
                                 TyrsZC2, RatesZC2, a2, sig2, y, ...
                                 TyrsFull, ht, m, T, S1, X1, S2, X2, rho)
% Calculate zero bond option price ZBC(t,T,S,X) under HW1F with a & (t)

% P(t,T) = exp{A(t,T) - x(t)*B(t,T)}
% TdysZC, RatesZC : Zero Curve
% a : Mean reversion
% sig : (t) vector
% x : x(i)
% TyrsFull : Time vector
% ht : dt vector
% m : Calculation time index. t=TyrsFull(m).
% T : Maturity of option
% S : Maturity of underlying zero bond (T<S)

% Numerical Integration
Integral1 = 0;
Integral2 = 0;
n = getIndex(TyrsFull, m, T);  % smallest index n s.t. TyrsFull(n)T.
for k=m:n-2
    Integral1 = Integral1 + sig1(k+1)^2 ...
               *(exp(2*a1*TyrsFull(k))+exp(2*a1*TyrsFull(k+1)))/2;
    Integral2 = Integral2 + sig2(k+1)^2 ...
               *(exp(2*a2*TyrsFull(k))+exp(2*a2*TyrsFull(k+1)))/2;
end
Integral1 = Integral1 + sig1(n)^2*(exp(2*a1*TyrsFull(n-1))+exp(2*a1*T))/2;
Integral2 = Integral2 + sig2(n)^2*(exp(2*a2*TyrsFull(n-1))+exp(2*a2*T))/2;

% Input Variables
sig1T = sqrt((exp(-a1*S1)-exp(-a1*T))^2*Integral1);
P1T = getZCB_HW1F(TyrsZC1, RatesZC1, a1, sig1, x, TyrsFull, ht, m, T);
P1S1 = getZCB_HW1F(TyrsZC1, RatesZC1, a1, sig1, x, TyrsFull, ht, m, S1);
d1_1 = (log(P1S1/P1T)-log(X1))/sig1T + sig1T/2;
d1_2 = d1_1 - sig1T;

sig2T = sqrt((exp(-a2*S2)-exp(-a2*T))^2*Integral2);
P2T = getZCB_HW1F(TyrsZC2, RatesZC2, a2, sig2, y, TyrsFull, ht, m, T);
P2S2 = getZCB_HW1F(TyrsZC2, RatesZC2, a2, sig2, y, TyrsFull, ht, m, S2);
d2_1 = (log(P2S2/P2T)-log(X2))/sig2T + sig2T/2;
d2_2 = d2_1 - sig2T;

% Option Value
ZBDC2d = P1T*mvncdf([d1_2 d2_2], [0, 0], [1 rho; rho 1]);