% Calculate zero bond price P(t,T) under HW1F with a & (t)
% dr(t) = [(t) + a*r(t)]dt + (t)*dW(t)

% P(t,T) = exp{A(t,T) - x(t)*B(t,T)}
function result = getZCB_HW1F(TyrsZC, RatesZC, a, sig, x, TyrsFull, ht, ...
                  m, T2)
% TdysZC, RatesZC : Zero Curve
% a : Mean reversion
% sig : (t) vector
% x : x(i)
% TyrsFull : Time vector
% ht : dt vector
% m : Calculation time index. t=TyrsFull(m).
% T2 : Maturity of zero bond. T=T2.

% B(t,T)
T1 = TyrsFull(m);
B = getBT_HW1F(a, T1, T2);

% A(t,T)
A = getAT_HW1F(TyrsZC, RatesZC, a, sig, TyrsFull, ht, m, T2);

% P(t,T)
result = exp(A - B*x);

% A(t,T) = -(u)d(u) + (1/2)*V(t,T)
function result = getAT_HW1F(TyrsZC, RatesZC, a, sig, TyrsFull, ht, m, T2)
% TdysZC, RatesZC : Zero Curve
% a : Mean reversion
% sig : (t) vector on TyrsFull
% TyrsFull : Time vector
% ht : dt vector
% m : Calculation time index. t=TyrsFull(m)
% T2 : Maturity of zero bond. T=T2

%(u)d(u)
T1 = TyrsFull(m);   % t
RT1 = interpol_Lin(TyrsZC, RatesZC, T1);  % P(0,t)
RT2 = interpol_Lin(TyrsZC, RatesZC, T2);  % P(0,T)
% disp('phi_T1T1');
phi_T1T1 = getPhi(a, sig, TyrsFull, ht, 1, T1, T1);  % (0,t,t)
% disp('phi_T2T2');
phi_T2T2 = getPhi(a, sig, TyrsFull, ht, 1, T2, T2);  % (0,T,T)
% disp('Integral');
Integral = (RT2*T2 - RT1*T1) + (phi_T2T2 - phi_T1T1)/2; % (u)d(u)

% V(t,T)
V = getPhi(a, sig, TyrsFull, ht, m, T2, T2);  % (t,T,T)

% A(t,T)
result = -Integral + V/2;

% (Ta,Tb,T)
function result = getPhi(a, sig, TyrsFull, ht, m, Tb, T)
% Ta = TyrsFull(m)

n = getIndex(TyrsFull, m, Tb);  % smallest index n s.t. TyrsFull(n)Tb.

if n==1
    tmpResult = 0;
else
     % B(t,T)
    BT_Square = zeros(n-1,1);
    for k=m:n-1 % u=t(m)~t(n-1)
        BT_Square(k) = getBT_HW1F(a, TyrsFull(k), T)^2;
    end

    % (Ta=TyrsFull(m),Tb,T) 
    tmpResult = 0;
    for k=m:n-2
        tmpResult = tmpResult + sig(k+1)^2 ...
                   *(BT_Square(k)+BT_Square(k+1))/2*ht(k);
    end
    tmpResult = tmpResult + sig(n)^2*BT_Square(n-1)/2*(T-TyrsFull(n-1));
end

result = tmpResult;
