function result = getZCB_HW1F0(TyrsZC, RatesZC, f0, a, sig, x, ...
                               TyrsFull, m, T2)
% Calculate zero bond price P(t,T) under HW1F with a & 
% dr(t) = [(t) + a*r(t)]dt + *dW(t)
% Formulas in page 76 in Brigo&Mercurio
%
% TdysZC, RatesZC : Zero Curve
% f0 : Mkt Observed instantaneous fwd curve
% a : Mean reversion
% sig : 
% x : x(i)
% TyrsFull : Time vector
% m : Calculation time index. t=TyrsFull(m).
% T2 : Maturity of zero bond. T=T2.

T1 = TyrsFull(m);
sigAvg = mean(sig);
% B(t,T)
B = getBT_HW1F(a, T1, T2);

% A(t,T)
DF1 = getDF(TyrsZC, RatesZC, T1);
DF2 = getDF(TyrsZC, RatesZC, T2);
A = (DF2/DF1)*exp( B*f0(m) - sigAvg^2/(4*a)*(1-exp(-2*a*T1))*B^2 );

% (t)
alpha0 = f0(m) + sigAvg^2/(2*a^2)*(1-exp(-a*T1))^2;

% P(t,T)
r = x + alpha0;
result = A*exp(-r*B);
